require "Docline\\MainDocline";

local Types = {};
Types["System.Object"] = luanet.import_type("System.Object");

function InjectScript(destination, script)
    local objArray = Types["System.Object"][2];
	objArray[0] = script;
	objArray[1] = "JavaScript";
	destination:InvokeScript("execScript", objArray);
end

function OnLoggedIn()
	local navRequestsElement = GetElementInFrame("doclinemenu", "navRequests")
	if navRequestsElement ~= nil then
		ClickObjectByReference(GetParentElement(navRequestsElement))
		NavRequestsClicked()
	end
end

function NavRequestsClicked()
	local borrowingElement = FindBorrowLink()
	if borrowingElement ~= nil then
		ClickObjectByReference(borrowingElement)
	end
end

function SearchFormLoaded()
    local element = FindPMIDInput()
    
    if (element ~= nil) then
        element.SetAttribute(element, "value", FieldTranslator.GetTranslationResults(FieldTranslator, "PubMedSearchID").Value)
    end
	
    local doclineFrame = GetFrameByName(nil, "mainframe")
	if doclineFrame == nil then
		return
	end	
    
    doclineFrame.Document:InvokeScript("submitFormPMID", nil);
end

function FindPMIDInput()
	local doclineFrame = GetFrameByName(nil, "mainframe")
	if doclineFrame == nil then
        return nil
	end	
    
    local inputCollection = doclineFrame.Document.GetElementsByTagName(doclineFrame.Document, "INPUT")
    local element = nil
	    
    for i = 0, inputCollection.Count - 1, 1 do	
        element = GetElementByCollectionIndex(inputCollection, i)
        if (element ~= nil and element.GetAttribute(element, "name") == "PMID") then
            return element
        end
    end
	
	return nil
end

function ResultsFormLoaded()
    local frame = GetFrameByName(nil, "editframe");
	
	--Inject a javascript function that invokes the submit function
	local script = "$(document).ready(function () { if (document.forms[0]['jorders0']) {selectAll(1); parent.subframe.LoadEditFrame('R');}})";

    InjectScript(frame.Document, script);

    StartPageWatcher(1000, 0);
end

function RouteFormLoaded()
    StopPageWatcher();
    
	-- We will need this later, so I'm going to grab it now so that it will already be cached when I do the SetFormValue calls.
	local routeForm = GetForm(GetFrameByName(nil, "editframe"), "theform")
	
	local currentValue = FieldTranslator.GetTranslationResults(FieldTranslator, "RequestForm").Value
	if (currentValue ~= nil) then
        local requestFormElements = routeForm.All:GetElementsByName("REQUEST_FORM");
       
        for i = 0, requestFormElements.Count - 1, 1 do
            input = GetElementByCollectionIndex(requestFormElements, i)
            
            if (input.GetAttribute(input, "value") == currentValue) then
                ClickObjectByReference(input);
                break;
            end
        end        
	end
    	
	currentValue = FieldTranslator.GetTranslationResults(FieldTranslator, "CopyrightCompliance").Value
	if currentValue ~= nil then
		SetFormValue("theform", "COPYRIGHT_COMPLIANCE", currentValue)
	end
	currentValue = FieldTranslator.GetTranslationResults(FieldTranslator, "ServiceLevel").Value
	if currentValue ~= nil then
		SetFormValue("theform", "SERVICE_LEVEL",currentValue )
	end
	currentValue = FieldTranslator.GetTranslationResults(FieldTranslator, "AuthorizedBy").Value
	if currentValue ~= nil then
		SetFormValue("theform", "REQUEST_AUTHORIZED_BY", currentValue)
	end
	currentValue = FieldTranslator.GetTranslationResults(FieldTranslator, "ReferralReason").Value
	if currentValue ~= nil then
		SetFormValue("theform", "REFERRAL_REASON", currentValue)
	end
	currentValue = FieldTranslator.GetTranslationResults(FieldTranslator, "MaxCost").Value
	if currentValue ~= nil then
		SetFormValue("theform", "MAX_COST", currentValue)
	end
	
	currentValue = FieldTranslator.GetTranslationResults(FieldTranslator, "PatronName").Value
	if currentValue ~= nil then
        SetFormValue("theform", "PNAME", currentValue)
    end

	currentValue = FieldTranslator.GetTranslationResults(FieldTranslator, "NetworkShippingOption").Value
	if currentValue ~= nil then
		SetFormValue("theform", "DELIVERY_NETWORK", currentValue)
	end
    
	currentValue = FieldTranslator.GetTranslationResults(FieldTranslator, "NLMShippingOption").Value
	if currentValue ~= nil then
		SetFormValue("theform", "DELIVERY_NLM", currentValue)
	end
    
	currentValue = FieldTranslator.GetTranslationResults(FieldTranslator, "PrefixLIBID").Value
	if currentValue ~= nil then
		SetFormValue("theform", "PREFIX_LIBID", currentValue)
	end
	
	currentValue = FieldTranslator.GetTranslationResults(FieldTranslator, "ShipToName").Value
	if currentValue ~= nil then
		SetFormValue("theform", "INST", currentValue)
	end
	currentValue = FieldTranslator.GetTranslationResults(FieldTranslator, "Department").Value
	if currentValue ~= nil then
		SetFormValue("theform", "DEPT", currentValue)
	end
	
    local frame = GetFrameByName(nil, "editframe");    
	local inputCollection = frame.Document:GetElementsByTagName("img");
	local element = nil;
    
	for i = 0, inputCollection.Count - 1, 1 do
		element = GetElementByCollectionIndex(inputCollection, i)
		if element.GetAttribute(element, "title") == "Finish" then			
            ClickObjectByReference(element.Parent)
			HandleManualPageChange()
			return
		end
	end
end

function AddressFormLoaded()
	--AddressFormSubmitButton.RaiseEvent(AddressFormSubmitButton, "onClick")
end

function Finished()
	StopPageWatcher(); --Ensure the Pages watcher is stopped to prevent looping

	local importFrame = nil
	local importRegex = nil
	local resultFormPage = false
	if GetForm(nil, "resultForm") ~= nil then
		importFrame = GetFrameByName(nil, "editframe")
		resultFormPage = true
	elseif GetForm(nil, "backForm") ~= nil then
		importFrame = GetFrameByName(nil, "mainframe")
		importRegex = [=[(?s:Result:(?<ID>\d+).*?\((?<Lender>.*?)\).*?(Part Author:(?<Authors>.*?\n)){0,1}(Part Title:(?<Article>.*?\n)){0,1}Citation:.*?(?<Year>\d{0,4}).*?;((?<Volume>.*?)\((?<Issue>.*?)\):(?<Pages>.*?\n)){0,1})]=]
	else
		importFrame = GetFrameByName(nil, "editframe")
		importRegex = StandardImportRegEx
	end	

	if (importFrame.Document.Body ~= nil) then
		local counter = 0
		while (importFrame.Document.Body.InnerText == "Working on requests, please wait..." and counter < 50) do
			Sleep(100)
			counter = counter + 1
		end
		
		if (counter == 50) then
			return
		end

		if resultFormPage == true then
			if string.find(importFrame.Document.Body.InnerText, "Result:", 0, true) ~= nil then
				importRegex = [=[(?s:Result:(?<ID>\d+).*?\((?<Lender>.*?)\).*?(Title:(?<Title>.*?\n)){0,1}(Author:(?<JournalAuthor>.*?\n)){0,1}(Part Title:(?<Article>.*?\n)){0,1}(Part Author:(?<Authors>.*?\n)){0,1}Citation:(?<Year>\d{0,4});(?<Volume>.*?)\((?<Issue>.*?)\):(?<Pages>.*?\n))]=]
			else
				importRegex = StandardImportRegEx
			end	
		end
		DoDataImport(importFrame.Document.Body.InnerText, importRegex)
		RegisterPageHandler("custom", "IsFinishedPage", "Finished", true)
	end
end

function FindBorrowLink()
	local doclineFrame = GetFrameByName(nil, "doclinemenu")
	if doclineFrame == nil then
		return nil
	end	
	local element = nil
	for i = 0, doclineFrame.Document.All.Count - 1, 1 do
		element = GetElementByWindowIndex(doclineFrame, i)
		if element.InnerHtml == "Borrow" then
			return element
		end
	end
	
	return nil
end

function FindElementByName(lWindow, lName)
	local element = nil
	for i = 0, lWindow.Document.All.Count - 1, 1 do
		element = GetElementByWindowIndex(lWindow, i)
		if element.Id == lName or element.Name == lName then
			return element
		end
	end
	
	local childFrameResult = nil
	
	for x = 0, lWindow.Frames.Count - 1, 1 do
		childFrameResult = FindElementByName(GetFrameByWindowIndex(lWindow, x), lName)
		if childFrameResult ~= nil then
			return childFrameResult
		end
	end
	
	return nil
end

RegisterPageHandler("formExists", "user", "LoginLoaded", false)
RegisterPageHandler("custom", "IsHomePage", "OnLoggedIn", false)
RegisterPageHandler("custom", "IsUniqueKeyPage", "SearchFormLoaded", false)
RegisterPageHandler("formExists", "myform", "ResultsFormLoaded", false)
RegisterPageHandler("formExists", "theform", "RouteFormLoaded", false)
RegisterPageHandler("custom", "IsAddressForm", "AddressFormLoaded", false)
RegisterPageHandler("custom", "IsFinishedPage", "Finished", true)

FieldTranslator = GetFieldTranslator()

Navigate(DoclineUrl);