require "Docline\\MainDocline";

function OnLoggedIn()
	local navRequestsElement = GetElementInFrame("doclinemenu", "navRequests")
	if navRequestsElement ~= nil then
		ClickObjectByReference(GetParentElement(navRequestsElement))
		NavRequestsClicked()
	end
end

function NavRequestsClicked()
	local borrowingElement = FindBorrowLink()
	if borrowingElement ~= nil then
		ClickObjectByReference(borrowingElement)
	end
end

function SearchFormLoaded()
    local element = FindPMIDInput()
    
    if (element ~= nil) then
        element.SetAttribute(element, "value", FieldTranslator.GetTranslationResults(FieldTranslator, "PubMedSearchID").Value)
    end
	
    local doclineFrame = GetFrameByName(nil, "mainframe")
	if doclineFrame == nil then
		return
	end	
    
    doclineFrame.Document:InvokeScript("submitFormPMID", nil);
end

function FindPMIDInput()
	local doclineFrame = GetFrameByName(nil, "mainframe")
	if doclineFrame == nil then
        return nil
	end	
    
    local inputCollection = doclineFrame.Document.GetElementsByTagName(doclineFrame.Document, "INPUT")
    local element = nil
	    
    for i = 0, inputCollection.Count - 1, 1 do	
        element = GetElementByCollectionIndex(inputCollection, i)
        if (element ~= nil and element.GetAttribute(element, "name") == "PMID") then
            return element
        end
    end
	
	return nil
end

function FindBorrowLink()
	local doclineFrame = GetFrameByName(nil, "doclinemenu")
	if doclineFrame == nil then
		return nil
	end	
	local element = nil
	for i = 0, doclineFrame.Document.All.Count - 1, 1 do
		element = GetElementByWindowIndex(doclineFrame, i)
		if element.InnerHtml == "Borrow" then
			return element
		end
	end
	
	return nil
end

function Finished()
	local importFrame = nil
	local importRegex = nil
	local resultFormPage = false
	if GetForm(nil, "resultForm") ~= nil then
		importFrame = GetFrameByName(nil, "editframe")
		resultFormPage = true
	elseif GetForm(nil, "backForm") ~= nil then
		importFrame = GetFrameByName(nil, "mainframe")
		importRegex = [=[(?s:Result:(?<ID>\d+).*?\((?<Lender>.*?)\).*?(Part Author:(?<Authors>.*?\n)){0,1}(Part Title:(?<Article>.*?\n)){0,1}Citation:.*?(?<Year>\d{0,4}).*?;((?<Volume>.*?)\((?<Issue>.*?)\):(?<Pages>.*?\n)){0,1})]=]
	else
		importFrame = GetFrameByName(nil, "editframe")
		importRegex = StandardImportRegEx
	end	

	if (importFrame.Document.Body ~= nil) then
		local counter = 0
		while (importFrame.Document.Body.InnerText == "Working on requests, please wait..." and counter < 50) do
			Sleep(100)
			counter = counter + 1
		end
		
		if (counter == 50) then
			return
		end

		if resultFormPage == true then
			if string.find(importFrame.Document.Body.InnerText, "Result:", 0, true) ~= nil then
				importRegex = [=[(?s:Result:(?<ID>\d+).*?\((?<Lender>.*?)\).*?(Title:(?<Title>.*?\n)){0,1}(Author:(?<JournalAuthor>.*?\n)){0,1}(Part Title:(?<Article>.*?\n)){0,1}(Part Author:(?<Authors>.*?\n)){0,1}Citation:(?<Year>\d{0,4});(?<Volume>.*?)\((?<Issue>.*?)\):(?<Pages>.*?\n))]=]
			else
				importRegex = StandardImportRegEx
			end	
		end
		DoDataImport(importFrame.Document.Body.InnerText, importRegex)
		RegisterPageHandler("custom", "IsFinishedPage", "Finished", true)
	end
end

RegisterPageHandler("formExists", "user", "LoginLoaded", false)
RegisterPageHandler("custom", "IsHomePage", "OnLoggedIn", false)
RegisterPageHandler("custom", "IsUniqueKeyPage", "SearchFormLoaded", false)
RegisterPageHandler("custom", "IsFinishedPage", "Finished", true)

FieldTranslator = GetFieldTranslator()

Navigate(DoclineUrl);